// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = point;         //sampler states
    MinFilter = point;
    MagFilter = point;
};

//texture
texture Tex2 <string uiname="Texture 2";>;
sampler Samp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = point;         //sampler states
    MinFilter = point;
    MagFilter = linear;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

#define PI 3.14159

float offset = 0.01;
float factor = 1;

float centerDist(float2 point, float4 col){

  return sin(distance(float2(0.5, 0.5), point*col)*PI*factor);
}

float4 PS_1(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, In.TexCd);
    float dist = centerDist(In.TexCd, col);
    float4 col2 = tex2D(Samp, float2(In.TexCd.x+offset*dist, In.TexCd.y+offset*dist));
    float4 col3 = tex2D(Samp, float2(In.TexCd.x-offset*dist, In.TexCd.y-offset*dist));
    col.rgb = (0.5*col.rgb)+(0.25*col2.rgb)+(0.25*col3.rgb);
    return col;
}

float4 PS_2(vs2ps In): COLOR
{
    float4 col = tex2D(Samp2, In.TexCd);
    float dist = centerDist(In.TexCd, col);
    float4 col2 = tex2D(Samp2, float2(In.TexCd.x-offset*dist, In.TexCd.y+offset*dist));
    float4 col3 = tex2D(Samp2, float2(In.TexCd.x+offset*dist, In.TexCd.y-offset*dist));
    col.rgb = (0.5*col.rgb)+(0.25*col2.rgb)+(0.25*col3.rgb);
    return col;
}

float4 PS2_1(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, In.TexCd);
    float dist = centerDist(In.TexCd, col);
    float4 col2 = tex2D(Samp, float2(In.TexCd.x+offset*dist, In.TexCd.y));
    float4 col3 = tex2D(Samp, float2(In.TexCd.x-offset*dist, In.TexCd.y));
    col.rgb = (0.5*col.rgb)+(0.25*col2.rgb)+(0.25*col3.rgb);
    return col;
}

float4 PS2_2(vs2ps In): COLOR
{
    float4 col = tex2D(Samp2, In.TexCd);
    float dist = centerDist(In.TexCd, col);
    float4 col2 = tex2D(Samp2, float2(In.TexCd.x, In.TexCd.y+offset*dist));
    float4 col3 = tex2D(Samp2, float2(In.TexCd.x, In.TexCd.y-offset*dist));
    col.rgb = (0.5*col.rgb)+(0.25*col2.rgb)+(0.25*col3.rgb);
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleShader
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_1();
    }
    
    pass P1
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_2();
    }
    
}

technique TSimpleShader2
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS2_1();
    }

    pass P1
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS2_2();
    }

}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
